import json
from unittest.mock import MagicMock

import httpx
import pytest

from arcade_dropbox.tools.files import download_file
from arcade_dropbox.utils import clean_dropbox_entry


@pytest.fixture
def file_content_response_header():
    return json.dumps({
        "id": "123",
        "name": "test.txt",
        "path_display": "/test.txt",
        "size": 1024,
        "server_modified": "2021-01-01T00:00:00Z",
        "content_hash": "1234567890",
        "is_downloadable": True,
        "rev": "a1c10ce0dd78",
        "sharing_info": {
            "modified_by": "dbid:AAH4f99T0taONIb-OurWxbNQ6ywGRopQngc",
            "parent_shared_folder_id": "84528192421",
            "read_only": True,
        },
    })


@pytest.mark.asyncio
async def test_download_file_success(
    mock_context,
    mock_httpx_client,
    file_content_response_header,
):
    file_content = "test file content"

    mock_httpx_response = MagicMock(spec=httpx.Response)
    mock_httpx_response.status_code = 200
    mock_httpx_response.json.side_effect = ValueError("not json")
    mock_httpx_response.headers = {"Dropbox-API-Result": file_content_response_header}
    mock_httpx_response.text = file_content

    mock_httpx_client.post.return_value = mock_httpx_response

    tool_response = await download_file(
        context=mock_context,
        file_path="test.txt",
    )

    expected_response = clean_dropbox_entry(json.loads(file_content_response_header))
    expected_response["content"] = file_content
    expected_response["type"] = "file"

    assert tool_response == {"file": expected_response}


@pytest.mark.asyncio
async def test_download_file_path_not_found(
    mock_context,
    mock_httpx_client,
):
    mock_httpx_response = MagicMock(spec=httpx.Response)
    mock_httpx_response.status_code = 409
    mock_httpx_response.json.return_value = {"error_summary": "path/not_found"}

    mock_httpx_client.post.return_value = mock_httpx_response

    tool_response = await download_file(
        context=mock_context,
        file_path="test.txt",
    )

    assert tool_response == {
        "error": "The specified path was not found by Dropbox",
    }


@pytest.mark.asyncio
async def test_download_file_unsupported_file(
    mock_context,
    mock_httpx_client,
):
    mock_httpx_response = MagicMock(spec=httpx.Response)
    mock_httpx_response.status_code = 409
    mock_httpx_response.json.return_value = {"error_summary": "unsupported_file/not_supported"}

    mock_httpx_client.post.return_value = mock_httpx_response

    tool_response = await download_file(
        context=mock_context,
        file_path="test.txt",
    )

    assert tool_response == {
        "error": "The specified file is not supported for the requested operation",
    }


@pytest.mark.asyncio
async def test_download_file_server_error(
    mock_context,
    mock_httpx_client,
):
    mock_httpx_response = MagicMock(spec=httpx.Response)
    mock_httpx_response.status_code = 500
    mock_httpx_response.text = "500 Internal server error"
    mock_httpx_response.json.side_effect = ValueError("not json")

    mock_httpx_client.post.return_value = mock_httpx_response

    tool_response = await download_file(
        context=mock_context,
        file_path="test.txt",
    )

    assert tool_response == {
        "error": "500 Internal server error",
    }



import os

import pytest
from arcadepy import Arcade


@pytest.fixture(scope="session")
def arcade_base_url():
    """
    Retrieve the ARCADE_BASE_URL from the environment, falling back to a default
    if not found.
    """
    return os.getenv("ARCADE_BASE_URL", "http://localhost:9099")


@pytest.fixture(scope="session")
def arcade_api_key():
    """
    Retrieve the ARCADE_API_KEY from the environment, falling back to a default
    if not found.
    """
    return os.getenv("ARCADE_API_KEY", "test_api_key")


@pytest.fixture(scope="session")
def arcade_client(arcade_base_url, arcade_api_key):
    """
    Creates a single Arcade client instance for use in all tests.
    Any method calls on this client can be patched/mocked within the tests.
    """
    client = Arcade(api_key=arcade_api_key, base_url=arcade_base_url)
    yield client
    # Teardown logic would go here if necessary



import random
from typing import Annotated

from arcade_tdk import tool


@tool
def generate_random_int(
    min_value: Annotated[str, "The minimum value of the random integer as a string"],
    max_value: Annotated[str, "The maximum value of the random integer as a string"],
    seed: Annotated[
        str | None,
        "The seed for the random number generator as a string."
        " If None, the current system time is used.",
    ] = None,
) -> Annotated[str, "A random integer between min_value and max_value as a string"]:
    """Generate a random integer between min_value and max_value (inclusive)."""
    if seed is not None:
        random.seed(int(seed))

    return str(random.randint(int(min_value), int(max_value)))  # noqa: S311


@tool
def generate_random_float(
    min_value: Annotated[str, "The minimum value of the random float as a string"],
    max_value: Annotated[str, "The maximum value of the random float as a string"],
    seed: Annotated[
        str | None,
        "The seed for the random number generator as a string."
        " If None, the current system time is used.",
    ] = None,
) -> Annotated[str, "A random float between min_value and max_value as a string"]:
    """Generate a random float between min_value and max_value."""
    if seed is not None:
        random.seed(int(seed))

    return str(random.uniform(float(min_value), float(max_value)))  # noqa: S311



from unittest.mock import AsyncMock, patch

import pytest

from arcade_google_drive.templates import optional_file_picker_instructions_template
from arcade_google_drive.tools import (
    get_file_tree_structure,
)
from arcade_google_drive.utils import build_drive_service


@pytest.fixture
def mock_context():
    context = AsyncMock()
    context.authorization.token = "mock_token"  # noqa: S105
    context.get_metadata.side_effect = lambda key: {
        "client_id": "123456789-abcdefg.apps.googleusercontent.com",
        "coordinator_url": "https://coordinator.example.com",
    }.get(key.value if hasattr(key, "value") else key)
    return context


@pytest.fixture
def mock_service():
    with patch(
        "arcade_google_drive.tools.drive." + build_drive_service.__name__
    ) as mock_build_service:
        yield mock_build_service.return_value


@pytest.mark.asyncio
async def test_get_file_tree_structure(
    mock_context, mock_service, sample_drive_file_tree_request_responses
):
    files_list_sample, drives_get_sample = sample_drive_file_tree_request_responses

    mock_service.files.return_value.list.return_value.execute.side_effect = [files_list_sample]
    mock_service.drives.return_value.get.return_value.execute.side_effect = drives_get_sample

    # Mock the generate_google_file_picker_url function
    with patch(
        "arcade_google_drive.tools.drive.generate_google_file_picker_url"
    ) as mock_file_picker:
        mock_file_picker.return_value = {
            "url": "https://coordinator.example.com/google/drive_picker?config=test_config",
            "llm_instructions": optional_file_picker_instructions_template.format(
                url="https://coordinator.example.com/google/drive_picker?config=test_config"
            ),
        }

        result = await get_file_tree_structure(mock_context, include_shared_drives=True)

    expected_file_tree = {
        "drives": [
            {
                "id": "0AFqcR6obkydtUk9PVA",
                "name": "Shared Drive 1",
                "children": [
                    {
                        "createdTime": "2025-02-26T00:27:45.526Z",
                        "id": "1dCOCdPxhTqiB3j3bWrIWM692ZbL8dyjt",
                        "mimeType": "application/vnd.google-apps.folder",
                        "modifiedTime": "2025-02-26T00:27:45.526Z",
                        "name": "shared-1-folder-1",
                        "children": [
                            {
                                "createdTime": "2025-02-26T00:28:20.571Z",
                                "id": "19WVyQndQsc0AxxfdrIt5CvDQd6r-BvpqnB8bWZoL7Xk",
                                "mimeType": "application/vnd.google-apps.document",
                                "modifiedTime": "2025-02-26T00:28:30.773Z",
                                "name": "shared-1-folder-1-doc-1",
                                "size": {
                                    "unit": "bytes",
                                    "value": 1024,
                                },
                            }
                        ],
                    },
                    {
                        "createdTime": "2025-02-26T00:27:19.287Z",
                        "id": "1didt_h-tDjuJ-dmYtHUSyOCPci30K_kSszvg0G3tKBM",
                        "mimeType": "application/vnd.google-apps.document",
                        "modifiedTime": "2025-02-26T00:27:26.079Z",
                        "name": "shared-1-doc-1",
                        "size": {
                            "unit": "bytes",
                            "value": 1024,
                        },
                    },
                ],
            },
            {
                "name": "My Drive",
                "children": [
                    {
                        "createdTime": "2025-01-24T06:34:22.305Z",
                        "id": "1vB6sv0MD0hYSraYvWU_fcci3GN_-Jf4g-LfyXdG8ZMo",
                        "mimeType": "application/vnd.google-apps.document",
                        "modifiedTime": "2025-02-25T21:54:30.632Z",
                        "name": "The Birth of MX Engineering",
                        "owners": [
                            {
                                "email": "one_new_tool_everyday@arcade.dev",
                                "name": "one_new_tool_everyday",
                            }
                        ],
                        "size": {
                            "unit": "bytes",
                            "value": 6634,
                        },
                    },
                    {
                        "createdTime": "2025-02-25T17:57:46.036Z",
                        "id": "1gqioaHG53jPVeJN5gBpHoO-GWtwiJcLo",
                        "mimeType": "application/vnd.google-apps.folder",
                        "modifiedTime": "2025-02-25T17:57:46.036Z",
                        "name": "test folder 1",
                        "owners": [
                            {
                                "email": "one_new_tool_everyday@arcade.dev",
                                "name": "one_new_tool_everyday",
                            }
                        ],
                        "children": [
                            {
                                "id": "1J92V9yvVWm_uNHq3CCY4wyG1H9B6iiwO",
                                "name": "test folder 1.1",
                                "mimeType": "application/vnd.google-apps.folder",
                                "createdTime": "2025-02-25T17:58:58.987Z",
                                "modifiedTime": "2025-02-25T17:58:58.987Z",
                                "owners": [
                                    {
                                        "email": "one_new_tool_everyday@arcade.dev",
                                        "name": "one_new_tool_everyday",
                                    }
                                ],
                                "children": [
                                    {
                                        "id": "1wv2dmYo0skJTI59ZIcwH9vm-wt7psMwXTvihuEGeHeI",
                                        "name": "test document 1.1.1",
                                        "mimeType": "application/vnd.google-apps.document",
                                        "createdTime": "2025-02-25T17:59:03.325Z",
                                        "modifiedTime": "2025-02-25T17:59:11.445Z",
                                        "owners": [
                                            {
                                                "email": "one_new_tool_everyday@arcade.dev",
                                                "name": "one_new_tool_everyday",
                                            }
                                        ],
                                        "size": {
                                            "unit": "bytes",
                                            "value": 1024,
                                        },
                                    },
                                ],
                            },
                            {
                                "id": "1DSmL7d07kjT6b6L-t4JIT06ElUbZ1q0K6_gEpn_UGZ8",
                                "name": "test document 1.2",
                                "mimeType": "application/vnd.google-apps.document",
                                "createdTime": "2025-02-25T17:58:38.628Z",
                                "modifiedTime": "2025-02-25T17:58:46.713Z",
                                "owners": [
                                    {
                                        "email": "one_new_tool_everyday@arcade.dev",
                                        "name": "one_new_tool_everyday",
                                    }
                                ],
                                "size": {
                                    "unit": "bytes",
                                    "value": 1024,
                                },
                            },
                            {
                                "id": "1Fcxz7HsyO2Zyc-5DTD3zBQnaVrZwD29BP9KD9rPnYfE",
                                "name": "test document 1.1",
                                "mimeType": "application/vnd.google-apps.document",
                                "createdTime": "2025-02-25T17:57:53.850Z",
                                "modifiedTime": "2025-02-25T17:58:28.745Z",
                                "owners": [
                                    {
                                        "email": "one_new_tool_everyday@arcade.dev",
                                        "name": "one_new_tool_everyday",
                                    }
                                ],
                                "size": {
                                    "unit": "bytes",
                                    "value": 1024,
                                },
                            },
                        ],
                    },
                    {
                        "createdTime": "2025-02-18T20:48:52.786Z",
                        "id": "16PUe97yGQeOjQgrgd54iCoxzid4SEvu_J33P_ELd5r8",
                        "mimeType": "application/vnd.google-apps.presentation",
                        "modifiedTime": "2025-02-19T23:31:20.483Z",
                        "name": "Hello world presentation",
                        "owners": [
                            {
                                "email": "john.doe@arcade.dev",
                                "name": "john.doe",
                            }
                        ],
                        "size": {
                            "unit": "bytes",
                            "value": 15774558,
                        },
                    },
                    {
                        "id": "1nG7lSvIyK05N9METPczVJa4iGgE7uoo-A6zpqjpUsDY",
                        "name": "Shared doc 1",
                        "mimeType": "application/vnd.google-apps.document",
                        "createdTime": "2025-02-19T18:51:44.622Z",
                        "modifiedTime": "2025-02-19T19:30:39.773Z",
                        "owners": [
                            {
                                "name": "theboss",
                                "email": "theboss@arcade.dev",
                            }
                        ],
                        "size": {
                            "unit": "bytes",
                            "value": 2700,
                        },
                    },
                ],
            },
        ],
        "file_picker": {
            "url": "https://coordinator.example.com/google/drive_picker?config=test_config",
            "llm_instructions": optional_file_picker_instructions_template.format(
                url="https://coordinator.example.com/google/drive_picker?config=test_config"
            ),
        },
    }

    assert result == expected_file_tree



from unittest.mock import patch

import pytest
from arcade_tdk import ToolContext, ToolSecretItem
from arcade_tdk.errors import ToolExecutionError

from arcade_firecrawl.tools import (
    cancel_crawl,
    crawl_website,
    get_crawl_data,
    get_crawl_status,
    map_website,
    scrape_url,
)


@pytest.fixture
def mock_context():
    return ToolContext(secrets=[ToolSecretItem(key="firecrawl_api_key", value="fake_api_key")])


@pytest.fixture
def mock_firecrawl_app_for_scrape():
    with patch("arcade_firecrawl.tools.scrape.FirecrawlApp") as app:
        yield app.return_value


@pytest.fixture
def mock_firecrawl_app_for_crawl():
    with patch("arcade_firecrawl.tools.crawl.FirecrawlApp") as app:
        yield app.return_value


@pytest.fixture
def mock_firecrawl_app_for_map():
    with patch("arcade_firecrawl.tools.map.FirecrawlApp") as app:
        yield app.return_value


@pytest.mark.asyncio
async def test_scrape_url_success(mock_firecrawl_app_for_scrape, mock_context):
    expected_response = {
        "success": True,
        "data": {"scraped_content": "scraped content"},
    }
    mock_firecrawl_app_for_scrape.scrape_url.return_value = expected_response

    result = await scrape_url(mock_context, "http://example.com")
    assert result == expected_response


@pytest.mark.asyncio
async def test_crawl_website_success(mock_firecrawl_app_for_crawl, mock_context):
    expected_response = {
        "id": "12345",
        "success": True,
    }
    mock_firecrawl_app_for_crawl.async_crawl_url.return_value = expected_response
    mock_firecrawl_app_for_crawl.check_crawl_status.return_value = expected_response

    result = await crawl_website(mock_context, "http://example.com")
    assert result == expected_response


@pytest.mark.asyncio
async def test_get_crawl_status_success(mock_firecrawl_app_for_crawl, mock_context):
    expected_response = {"status": "completed"}
    mock_firecrawl_app_for_crawl.check_crawl_status.return_value = expected_response

    result = await get_crawl_status(mock_context, "12345")
    assert result == expected_response


@pytest.mark.asyncio
async def test_get_crawl_data_success(mock_firecrawl_app_for_crawl, mock_context):
    expected_response = {"data": "crawl data"}
    mock_firecrawl_app_for_crawl.check_crawl_status.return_value = expected_response

    result = await get_crawl_data(mock_context, "12345")
    assert result == expected_response


@pytest.mark.asyncio
async def test_cancel_crawl_success(mock_firecrawl_app_for_crawl, mock_context):
    expected_response = {"status": "cancelled"}
    mock_firecrawl_app_for_crawl.cancel_crawl.return_value = expected_response

    result = await cancel_crawl(mock_context, "12345")
    assert result == expected_response


@pytest.mark.asyncio
async def test_map_website_success(mock_firecrawl_app_for_map, mock_context):
    expected_response = {"map": "website map"}
    mock_firecrawl_app_for_map.map_url.return_value = expected_response

    result = await map_website(mock_context, "http://example.com")
    assert result == expected_response


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "method,params,error_message",
    [
        (scrape_url, ("http://example.com",), "Error scraping URL"),
        (crawl_website, ("http://example.com",), "Error crawling website"),
        (get_crawl_status, ("12345",), "Error getting crawl status"),
        (get_crawl_data, ("12345",), "Error getting crawl data"),
        (cancel_crawl, ("12345",), "Error cancelling crawl"),
        (map_website, ("http://example.com",), "Error mapping website"),
    ],
)
async def test_firecrawl_error(
    mock_firecrawl_app_for_scrape,
    mock_firecrawl_app_for_crawl,
    mock_firecrawl_app_for_map,
    mock_context,
    method,
    params,
    error_message,
):
    mock_firecrawl_app_for_scrape.scrape_url.side_effect = Exception(error_message)
    mock_firecrawl_app_for_crawl.async_crawl_url.side_effect = Exception(error_message)
    mock_firecrawl_app_for_crawl.check_crawl_status.side_effect = Exception(error_message)
    mock_firecrawl_app_for_crawl.cancel_crawl.side_effect = Exception(error_message)
    mock_firecrawl_app_for_map.map_url.side_effect = Exception(error_message)

    with pytest.raises(ToolExecutionError):
        await method(mock_context, *params)



from unittest.mock import patch

import pytest
from arcade_tdk import ToolContext, ToolSecretItem
from arcade_tdk.errors import ToolExecutionError

from arcade_web.tools.firecrawl import (
    cancel_crawl,
    crawl_website,
    get_crawl_data,
    get_crawl_status,
    map_website,
    scrape_url,
)


@pytest.fixture
def mock_context():
    return ToolContext(secrets=[ToolSecretItem(key="firecrawl_api_key", value="fake_api_key")])


@pytest.fixture
def mock_firecrawl_app():
    with patch("arcade_web.tools.firecrawl.FirecrawlApp") as app:
        yield app.return_value


@pytest.mark.asyncio
async def test_scrape_url_success(mock_firecrawl_app, mock_context):
    mock_firecrawl_app.scrape_url.return_value = {"data": "scraped content"}

    result = await scrape_url(mock_context, "http://example.com")
    assert result == {"data": "scraped content"}


@pytest.mark.asyncio
async def test_crawl_website_success(mock_firecrawl_app, mock_context):
    mock_firecrawl_app.async_crawl_url.return_value = {"crawl_id": "12345"}

    result = await crawl_website(mock_context, "http://example.com")
    assert result == {"crawl_id": "12345"}


@pytest.mark.asyncio
async def test_get_crawl_status_success(mock_firecrawl_app, mock_context):
    mock_firecrawl_app.check_crawl_status.return_value = {"status": "completed"}

    result = await get_crawl_status(mock_context, "12345")
    assert result == {"status": "completed"}


@pytest.mark.asyncio
async def test_get_crawl_data_success(mock_firecrawl_app, mock_context):
    mock_firecrawl_app.check_crawl_status.return_value = {"data": "crawl data"}

    result = await get_crawl_data(mock_context, "12345")
    assert result == {"data": "crawl data"}


@pytest.mark.asyncio
async def test_cancel_crawl_success(mock_firecrawl_app, mock_context):
    mock_firecrawl_app.cancel_crawl.return_value = {"status": "cancelled"}

    result = await cancel_crawl(mock_context, "12345")
    assert result == {"status": "cancelled"}


@pytest.mark.asyncio
async def test_map_website_success(mock_firecrawl_app, mock_context):
    mock_firecrawl_app.map_url.return_value = {"map": "website map"}

    result = await map_website(mock_context, "http://example.com")
    assert result == {"map": "website map"}


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "method,params,error_message",
    [
        (scrape_url, ("http://example.com",), "Error scraping URL"),
        (crawl_website, ("http://example.com",), "Error crawling website"),
        (get_crawl_status, ("12345",), "Error getting crawl status"),
        (get_crawl_data, ("12345",), "Error getting crawl data"),
        (cancel_crawl, ("12345",), "Error cancelling crawl"),
        (map_website, ("http://example.com",), "Error mapping website"),
    ],
)
async def test_firecrawl_error(mock_firecrawl_app, mock_context, method, params, error_message):
    mock_firecrawl_app.scrape_url.side_effect = Exception(error_message)
    mock_firecrawl_app.async_crawl_url.side_effect = Exception(error_message)
    mock_firecrawl_app.check_crawl_status.side_effect = Exception(error_message)
    mock_firecrawl_app.cancel_crawl.side_effect = Exception(error_message)
    mock_firecrawl_app.map_url.side_effect = Exception(error_message)

    with pytest.raises(ToolExecutionError):
        await method(mock_context, *params)
